/*
This is the code for my Gasifier Monitor/Datalogger.  The device is normally connected to a Chinese JXQ-10A Gasifier which outputs woodgas to an Onan 6.5NH genset.  But lately, I have been using it as a shop tool to test and measure things.
See:
http://www.youtube.com/watch?v=RwoIPodGIQI
and:
http://www.spaco.org/Woodgas/thermocouple%20calibration/thermocouple%20calibration%20for%20woodgas%20reactor.htm
---------------------------------------------------------------------------

So many parts of this code are borrowed from other sources that it is hard to give credit where credit is really due.  Arduino.cc, Adafruit, Jeremy Blum are a few of the sources.

Pete Stanaitis  May 30, 2013
www.spaco.org
spaco@baldwin-telecom.net

See the system run at: 
http://www.youtube.com/watch?v=du9PXSoR3j0



***************** Revision T ********************************
5/5/2013, OOAM  Adding code for reading O2 sensor
Using pin A9.  This code has not been tested yet.

****New Features since Rev 0*****
There are now 8 pressure sensors
Name, range and gasifier-to-electric application of the pressure sensors:
(All sensors are Freescale MPXV****DP, or something pretty close)
Note1: 7002 = 8.03"wc,7007 = 28.105"wc, 5010 = 40.15"wc, 5050 = 200.75"wc
Note2: 7002 = 2Kpa, 7007 = 7 Kpa, 5010 = 10 Kpa, 5050 = 50 Kpa
Note3: 7002 = 0.3 Psi, 7007 = 1 Psi, 5010 = 1.45 Psi, 5050 = 7.25 Psi
Pr1 2Kpa_1, below grate
Pr2 2Kpa_2, center chamber
Pr3, 7Kpa_1, Right Chamber
Pr4, 7Kpa_2, differential mode across the final filter that is yet to be made
Pr5, 10Kpa_1, engine mixer sensor
Pr6, 7Kpa_3, Spare for now
Pr7, 7Kpa_4,  Spare for now
Pr8, 50Kpa_1, Spare for now
----------------------------------------------------------------

***************************Code snippet sequence:*************************
*******************Startup Section****************
Library "includes"
T/C pin assignments
Thermistor assigments
Pressure pin assignments
Marker button stuff
SD card logger stuff and some left over RTC
****Void Setup Section***********
Marker
LCD
T/C's
SD and auto log file naming
Log Header for SD card and serial monitor
********Void Loop Section***********
Marker
LCD
Time calculations (some old RTC code could be eliminated)
Thermistor, read, map, log, print to serial and lcd
Log and print T/C's to file and serial monitor
Pressure sensors, read, map, print to lcd, ser. mon and log
Calculate and log, etc., the  Vcc voltage.  It could be done better.
Blink green, then red leds to show reads are done and then write is done
Send Pressure readings to lcd
**************End of Program*************************


   The LCD circuit:
 * LCD RS pin to digital pin 7
 * LCD Enable pin to digital pin 8
 * LCD D4 pin to digital pin 9
 * LCD D5 pin to digital pin 10
 * LCD D6 pin to digital pin 11
 * LCD D7 pin to digital pin 12
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)
 *******************************************
 
 */
//****************************************************************
// include the library code:
#include <LiquidCrystal.h>
//includes for "lighttemplogger":
#include <SD.h>
#include <Wire.h>
#include "RTClib.h"
#include "max6675.h"

// initialize the LCD library with the numbers of the interface pins
LiquidCrystal lcd(7, 8, 9, 10, 11, 12);

//TrafficLightB (below grate temp)  assignments
//Note that there are other red and green leds, but not with
//same exact names
const int redLed  = 22;
const int grnLed = 23;
const int yelLed = 24;
int testTC = 0;

//Traffic Lights assignments for below grate pressure
const int orangeLed  = 25;
const int blueLed = 26;
const int whiteLed = 27;
float testPR = 0;


//Thermocouple pin assignments
//TC1
int thermoDO_1 = 44;
int thermoCS_1 = 45;
int thermoCLK_1= 46;
int th1;

//TC2
int thermoDO_2 = 47;
int thermoCS_2 = 48;
int thermoCLK_2= 49;
int th2;

MAX6675 thermocouple_1(thermoCLK_1, thermoCS_1, thermoDO_1);
MAX6675 thermocouple_2(thermoCLK_2, thermoCS_2, thermoDO_2);

// Thermistor analog pin
#define tempPin 0               
int temperatureF = 0;


//Pressure sensor assignments
int press1Pin =  A1; //This is a change from A0 to A1
int press2Kpa_1;
int press2Pin =  A2;
int press2Kpa_2;
int press3Pin =  A3;
int press7Kpa_1;
int press4Pin =  A4;
int press7Kpa_2;
int press5Pin = A5;
int press10Kpa_1;
int press6Pin = A6;
int press7Kpa_3;
int press7Pin = A7;
int press7Kpa_4;
int press8Pin = A8;
int press50Kpa_1;

//Pressure Traffic Lights Variables assignments
float p1sum = 0; 
float p1average = 0;

//Marker assignments
int switchPin = 6;
int ledPin = 5;
int marker = 0;

//define stuff for the SD card Logger and the LED indicators
#define LOG_INTERVAL  5000 // mills between entries (reduce to take more/faster data)
#define SYNC_INTERVAL 1000 // mills between calls to flush() - to write data to the card
uint32_t syncTime = 0; // time of last sync()

#define ECHO_TO_SERIAL   1 // echo data to serial port
#define WAIT_TO_START    0 // Wait for serial input in setup()

// the digital pins that connect to the LEDs
#define redLEDpin 2
#define greenLEDpin 3

//Code to prepare reference voltage to read and log
//I am not sure it is needed, or even, exactly what it does.
#define BANDGAPREF 14            // special indicator that we want to measure the bandgap
#define aref_voltage 3.3         // we tie 3.3V to ARef and measure it with a multimeter!
#define bandgap_voltage 1.1      // this is not super guaranteed but its not -too- off

//Don't need this, but have to carefully remove references to it
//before removing this line of code
RTC_DS1307 RTC; // define the Real Time Clock object

// For the data logging shield, we use digital pin 53 
//for the SD cs line, because this is for ATMega2560
const int chipSelect = 53;


// the logging file
File logfile;

void error(char *str)
{
  Serial.print("error: ");
  Serial.println(str);
  
  // red LED indicates error if it is "on" all the time
  digitalWrite(redLEDpin, HIGH);

  while(1);
}

//***********************VOID SETUP ****************************
void setup()
{
//Void Setup stuff for Traffic Temperature Lights
  {
   pinMode(redLed, OUTPUT);
   pinMode(grnLed, OUTPUT);
   pinMode(yelLed, OUTPUT);
  }
 
 //Void Setup stuff for Traffic Pressure Lights
 {
pinMode(orangeLed, OUTPUT);
pinMode(blueLed, OUTPUT);
pinMode(whiteLed, OUTPUT);
 }
  
  
  //Void Setup stuff for Marker
{
  pinMode(switchPin, INPUT);
  pinMode(ledPin, OUTPUT);
}

  
//Void Setup LCD Stuff  
{
  // set up the LCD's number of columns and rows: 
  lcd.begin(20, 4);
  
 
}

//Void setup stuff for TC's
 Serial.println("MAX6675 test");
  // wait for MAX chip to stabilize
  delay(500);


//Void setup SD Card Stuff
  Serial.begin(9600);
  Serial.println();
  
  // Assign pins for  debugging LEDs
  pinMode(redLEDpin, OUTPUT);
  pinMode(greenLEDpin, OUTPUT);
  
#if WAIT_TO_START
  Serial.println("Type any character to start");
  while (!Serial.available());
#endif //WAIT_TO_START

  // initialize the SD card
  Serial.print("Initializing SD card...");
  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(53, OUTPUT);
  
  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) 
  
  {
    error("Card failed, or not present");
  }
  
  Serial.println("card initialized.");
  
  // create a new file
  char filename[] = "LOGGER00.CSV";
  for (uint8_t i = 0; i < 100; i++)
 
  {
    filename[6] = i/10 + '0';
    filename[7] = i%10 + '0';
    if (! SD.exists(filename))
   
    {
      // only open a new file if it doesn't exist
      logfile = SD.open(filename, FILE_WRITE); 
      break;  // leave the loop!
    }
  }
  
  if (! logfile)
  {
    error("couldnt create file");
  }
  
  Serial.print("Logging to: ");
  Serial.println(filename);

  // connect to RTC
  Wire.begin();  
  if (!RTC.begin())
 
  {
    logfile.println("RTC failed");
#if ECHO_TO_SERIAL
    Serial.println("RTC failed");
#endif  //ECHO_TO_SERIAL
  }
  
//Log Header to SD CARD and to Serial Monitor
  logfile.println("Sec,AmT,TC1,TC2,BlwGr,CtrCh,RtCh,Fltr,Eng,Pr6-7007,Pr7-7007,Pr8-5050,Marker");    
#if ECHO_TO_SERIAL
  Serial.println("Sec,AmT,TC1,   TC2, BlwGr,PrLts,CtrCh, RtCh, Fltr, Eng,P6-07, P7-07, P8-50, Marker");
#endif //ECHO_TO_SERIAL
//End of SD Card Void Setup Stuff
 
}

//*****************************Void Loop***********************************
//*************************************************************************
void loop()
{
//Void Loop stuff for Marker  
 {
  if (digitalRead(switchPin) == HIGH)
  {
    digitalWrite(ledPin, HIGH);
    marker = marker + 1;
    lcd.setCursor(17, 3);
    lcd.print(marker);
    
  }
  else
  {
    digitalWrite(ledPin, LOW);
  }
} 
  
  
  
//Void Loop Stuff for LCD  
{
  // set the cursor to column 0, line 1
  //(note: line 1 is the 2nd row, since counting begins with 0)
  //Format is: (character position, row)
  lcd.setCursor(11, 0);
 lcd.print("         ");  
  lcd.setCursor(9, 0);
   
  // print TC 1 and TC 2
  lcd.print("TA");
  thermocouple_1.readFahrenheit();
   th1 = thermocouple_1.readFahrenheit();  
  lcd.print(th1);
  lcd.setCursor(15, 0);
  lcd.print("B");
  thermocouple_2.readFahrenheit();  
  th2 = thermocouple_2.readFahrenheit();
  lcd.print(th2);
  
 lcd.setCursor(0, 0);
  // print the number of minutes since reset:
  lcd.print("M"); 
  lcd.print(millis()/60000);
  
 
  
  lcd.setCursor(14, 3);
  lcd.print("Mkr");
}

// Void Loop Stuff for SD Card
{
  DateTime now;

  // delay for the amount of time we want between readings
  delay((LOG_INTERVAL -1) - (millis() % LOG_INTERVAL));
  
  digitalWrite(greenLEDpin, HIGH);
  int tempsec =0;
  
  // log milliseconds since starting
  uint32_t m = millis();
  tempsec =(millis() / 1000);
  logfile.print(tempsec);           // milliseconds since start
  logfile.print(", ");    
#if ECHO_TO_SERIAL
  Serial.print(tempsec);         // milliseconds since start
  Serial.print(",  ");  
#endif

  // fetch the time
  now = RTC.now();
 //I should get rid of this RTC stuff.


 
//Read Thermistor and map it to Pete's estimate
  analogRead(tempPin); 
  delay(10);
  int val = analogRead(tempPin);    
  int temperatureF = map(val, 0, 1023, 0, 150);
 
 //Add Thermistor reading to SD card data string 
  //logfile.print(", ");    
  logfile.print(temperatureF);


//Set LCD cursor to position thermistor 
  lcd.setCursor(4, 0);
  lcd.print("AT");
//Print Thermistor reading to lcd and to serial monitor  
lcd.setCursor(6, 0);
//lcd.print(" ");
  lcd.print(temperatureF);
  
  
#if ECHO_TO_SERIAL
  Serial.print(temperatureF);
#endif //ECHO_TO_SERIAL

// TC logging stuff in here
 
 //Void loop stuff for LOGGING TC's (not using Celcius)
   
   logfile.print(",");
   logfile.print(th1);
   Serial.print(", ");
   Serial.print(th1);
   
 
   logfile.print(",");
   logfile.print(th2);
 Serial.print(", ");
   Serial.print(th2);
//End of TC logging stuff 

//TrafficLight, getting the current  reading
  testTC = thermocouple_2.readFahrenheit(); 

  //*************Void loop stuff for Pressure Sensors****************
  
 //Clear pressure reading on the LCD screen
  lcd.setCursor(0, 1);
  lcd.print("                    ");
  lcd.setCursor(0, 2);
  lcd.print("                    ");
  lcd.setCursor(0, 3);
  lcd.print("              ");
  
//*****************Pressure Sensors*************************
//PR1, Below Grate
analogRead(press1Pin); 
  delay(10);
//Get pressure, map and send to serial monitor
 int p1val = analogRead(press1Pin);   
    float p1temp = map(p1val, 0, 1023, 0, 280);
    p1temp = (p1temp - 143);
   float press2Kpa_1 = (p1temp /13.875);
   Serial.print(", "); 
   Serial.print(press2Kpa_1);
//Log the press_1
logfile.print(", ");
  logfile.print(press2Kpa_1);
  
//Pressure Traffic Lights code
//***********************************************************
//Here's where I  take 3 readings and average them


//Here's where it starts:
{
for (int i=0; i <= 2; i++)
{
 analogRead(press1Pin); 
  delay(1000);
//Get pressure, map and send to serial monitor
 int p1val = analogRead(press1Pin);   
    float p1temp = map(p1val, 0, 1023, 0, 280);
    p1temp = (p1temp - 143);
   float press2Kpa_1 = (p1temp /13.875); 
  
  

  p1sum =  p1sum + press2Kpa_1; 
  Serial.print(p1sum);
   Serial.print(", ");    
 } 
 //take the average of the 3 readings 
{ 
 p1average = p1sum/3;
 testPR = p1average;
 Serial.print(", ");
 Serial.print(testPR);
 p1sum = 0; 
 p1average = 0;
}
}

//if it is in the below range, then set
//all Led's to Off
//below range: 
if ((testPR >= -.3) && (testPR <= .3))
   {
   digitalWrite(orangeLed, LOW);
   digitalWrite(blueLed, LOW);
   digitalWrite(whiteLed, LOW);
   }
    
//low range:
  if ((testPR  >= -.8)  && (testPR <= -.4))
  {
     digitalWrite(orangeLed, HIGH);
  }
  else{
     digitalWrite(orangeLed, LOW);
  }

//Right on range
  if ((testPR >= -2.5) && (testPR <= -.8))
   {
     digitalWrite(blueLed, HIGH);
   }  
   else
   {
     digitalWrite(blueLed, LOW);
   }
   
//Too High range
   if ((testPR >= -8) && (testPR <= -2.5))
   {
     digitalWrite(whiteLed, HIGH);
   }
   else
   {
     digitalWrite(whiteLed, LOW);
   }

//End of Pressure Traffic Lights code
//***************************************************************

//PR2, Center Chamber
   analogRead(press2Pin); 
  delay(10);
//Get pressure, map and send to serial monitor 
 int p2val = analogRead(press2Pin);   
    float p2temp = map(p2val, 0, 1023, 0, 280);
    p2temp = (p2temp - 144);
   float press2Kpa_2 = (p2temp /13.875);
   Serial.print(", "); 
   Serial.print(press2Kpa_2);
//Log the press_2
logfile.print(", ");
  logfile.print(press2Kpa_2); 

//PR3,, Right Chamber
   analogRead(press3Pin); 
  delay(10);
//Get pressure, map and send to serial monitor 
 int p3val = analogRead(press3Pin);   
    float p3temp = map(p3val, 0, 1023, 0, 560);
    p3temp = (p3temp - 283);
   float press7Kpa_1 = (p3temp /8);
   Serial.print(", "); 
   Serial.print(press7Kpa_1);
//Log the press_3
logfile.print(", ");
  logfile.print(press7Kpa_1);

//Pr4, Differential mode across the final filter that is yet to be made
    analogRead(press4Pin); 
  delay(10);
//Get pressure, map and send to serial monitor 
 int p4val = analogRead(press4Pin);   
    float p4temp = map(p4val, 0, 1023, 0, 560);
    p4temp = (p4temp - 282);
   float press7Kpa_2 = (p4temp /8);
   Serial.print(", "); 
   Serial.print(press7Kpa_2);
//Log the press_4
logfile.print(", ");
  logfile.print(press7Kpa_2);  
  

//Pr5, Engine Mixer Sensor, 10Kpa_1, engine mixer sensor
    analogRead(press5Pin); 
  delay(30);
//Get pressure, map and send to serial monitor 
 int p5val = analogRead(press5Pin);   
    float p5temp = map(p5val, 0, 1023, 0, 705);
    p5temp = (p5temp - 28);
   float press10Kpa_1 = (p5temp /-16);
   Serial.print(", "); 
   Serial.print(press10Kpa_1);
//Log the press_5
logfile.print(", ");
  logfile.print(press10Kpa_1);  


//Pr6, 7Kpa_3, First Spare for now
    analogRead(press6Pin); 
  delay(10);
//Get pressure, map and send to serial monitor 
 int p6val = analogRead(press6Pin);   
    float p6temp = map(p6val, 0, 1023, 0, 560);
    p6temp = (p6temp - 285);
   float press7Kpa_3 = (p6temp /8);
   Serial.print(", "); 
   Serial.print(press7Kpa_3);
//Log the press_6
logfile.print(", ");
  logfile.print(press7Kpa_3);  
  
//Pr7, 7Kpa_4, Second Spare for now
    analogRead(press7Pin); 
  delay(10);
//Get pressure, map and send to serial monitor 
 int p7val = analogRead(press7Pin);   
    float p7temp = map(p7val, 0, 1023, 0, 560);
    p7temp = (p7temp - 282);
   float press7Kpa_4 = (p7temp /8);
   Serial.print(", "); 
   Serial.print(press7Kpa_4);
//Log the press_7
logfile.print(", ");
  logfile.print(press7Kpa_4);  
  
//Pr8, 50Kpa_1, Second Spare for now
    analogRead(press8Pin); 
  delay(10);
//Get pressure, map and send to serial monitor 
 int p8val = analogRead(press8Pin);   
    float p8temp = map(p8val, 0, 1023, 0, 1100);
    p8temp = (p8temp - 53);
   float press50Kpa_1 = (p8temp /-5);
   Serial.print(", "); 
   Serial.print(press50Kpa_1);
//Log the press_8
logfile.print(", ");
  logfile.print(press50Kpa_1);   
    

 //*****************End of Pressure Sensor Code******************************** 
  

  // Log the estimated 'VCC' voltage by measuring the internal 1.1v ref
// This is an artifact from a previous program. Not sure why I left it in here
  analogRead(BANDGAPREF); 
  delay(10);
  int refReading = analogRead(BANDGAPREF); 
  float supplyvoltage = (bandgap_voltage * 1024) / refReading; 
  
  logfile.print(", ");
  //logfile.print(supplyvoltage);
  //logfile.print(", ");
  logfile.print(marker);
  
#if ECHO_TO_SERIAL
  Serial.print(", ");   
  //Serial.print(supplyvoltage);
  //Serial.print(", ");
  Serial.print(marker);
#endif // ECHO_TO_SERIAL

  logfile.println();
#if ECHO_TO_SERIAL
  Serial.println();
#endif // ECHO_TO_SERIAL

  //Show that we completed the "read" cycle
  digitalWrite(greenLEDpin, LOW);
  delay(100);
  // Now we write data to SD card! Don't sync too often - 
  // requires 2048 bytes of I/O to SD card
  // which uses a bunch of power and takes time
  if ((millis() - syncTime) < SYNC_INTERVAL) return;
  syncTime = millis();
  
  // blink  RED LED to show we are syncing data to the card & updating FAT!
  digitalWrite(redLEDpin, HIGH);
  logfile.flush();
  digitalWrite(redLEDpin, LOW);
  
 //Send TC1, TC2 and Ambient Temp Readings, Pressure Readings and Marker to LCD  
//Print Press_1 to LCD  This should be Below Grate, left on the upper pressure row
  lcd.setCursor(0, 1);
  //lcd.print("P1 ");
  lcd.print(press2Kpa_1, 1);
  
 //Print Press_2 to LCD  This should be the center chamber, center of LCD upper row
  lcd.print("  ");
  lcd.print(press2Kpa_2, 1);
  
  //Print Press_3 to LCD  This should be right chamber, right side of upper pressure roow
  lcd.print("  ");
  lcd.print(press7Kpa_1, 1);
  
  //Print Press_4 to LCD  This is the left one on the bottom row
  lcd.setCursor(0, 2);
 // lcd.print("  ");
  lcd.print(press7Kpa_2, 1);
  
  //Print Press_5 to LCD  This should be final filter, ???
  lcd.print("  ");
  lcd.print(press10Kpa_1, 1);
  
  //Print Press_6 to LCD  This is the spare.  ???
  lcd.print("  ");
  lcd.print(press7Kpa_3, 1);

  //Print Press_7 to LCD  This is the spare.  ???
  lcd.setCursor(0, 3);
  lcd.print(press7Kpa_4, 1);
  
  //Print Press_8 to LCD  This is the spare.  ???
  lcd.setCursor(7, 3);
  lcd.print(" ");
  lcd.print(press50Kpa_1, 1);
  
  //************Below Grate Temperature TrafficLight Code**************

 //if T/C_2 is in the below range, make sure all traffic lights are OFF
 //Below Range: 
 {  
  if ((testTC >= 0) && (testTC <= 450))
   {
   digitalWrite(redLed, LOW);
   digitalWrite(grnLed, LOW);
   digitalWrite(yelLed, LOW);
   }

   //Low Range:
  if ((testTC >=451)  && (testTC <=800))
  {
     digitalWrite(redLed, HIGH);
  }
  else
  {
     digitalWrite(redLed, LOW);
  }

  //Right-On (Ideal)Range
  if ((testTC >=801) && (testTC <=1200))
   {
     digitalWrite(grnLed, HIGH);
   }  
   else
   {
     digitalWrite(grnLed, LOW);
   }
   
   //Too High range
   if ((testTC >=1201) && (testTC <=2300))
   {
     digitalWrite(yelLed, HIGH);
   }
   else
   {
     digitalWrite(yelLed, LOW);
   }

 }
//Pressure Traffic Lights code ends here     
}

//Oxygen Sensor Code

  int OxysensorValue = analogRead(A9);
  float OxyVout = OxysensorValue * (5.0 / 1023);
  Serial.println(OxyVout);
}